@@ -131,7 +131,8 @@ void cram_stats_dump(cram_stats *st) {
  * Returns the best codec to use.
  */
 enum cram_encoding cram_stats_encoding(cram_fd *fd, cram_stats *st) {
-    int nvals, i, ntot = 0, max_val = 0, min_val = INT_MAX;
+    int nvals, i, ntot = 0;
+    int64_t max_val = 0, min_val = INT64_MAX;
     int *vals = NULL, *freqs = NULL, vals_alloc = 0;
 
     //cram_stats_dump(st);
@@ -160,7 +161,7 @@ enum cram_encoding cram_stats_encoding(cram_fd *fd, cram_stats *st) {
     if (st->h) {
 	HashIter *iter=  HashTableIterCreate();
 	HashItem *hi;
-	int i;
+	int64_t i;
 
 	while ((hi = HashTableIterNext(st->h, iter))) {
 	    if (nvals >= vals_alloc) {
@@ -182,12 +183,27 @@ enum cram_encoding cram_stats_encoding(cram_fd *fd, cram_stats *st) {
     }
 
     st->nvals = nvals;
+    st->min_val = min_val;
+    st->max_val = max_val;
     assert(ntot == st->nsamp);
     free(vals);
     free(freqs);
 
+    if (fd->verbose > 1)
+	fprintf(stderr, "Range = %"PRId64"..%"PRId64", nvals=%d, ntot=%d\n",
+		min_val, max_val, nvals, ntot);
+
     // Crude and simple alternative.
-    return nvals > 1 ? E_EXTERNAL : E_HUFFMAN;
+    if (CRAM_MAJOR_VERS(fd->version) >= 4) {
+	if (nvals == 1)
+	    return E_CONST_INT;
+	else if (nvals == 0 || min_val < 0)
+	    return E_VARINT_SIGNED;
+	else
+	    return E_VARINT_UNSIGNED;
+    } else {
+	return nvals != 1 ? E_EXTERNAL : E_HUFFMAN;
+    }
 
 
 #ifdef RANDOMISER
@@ -209,10 +225,6 @@ enum cram_encoding cram_stats_encoding(cram_fd *fd, cram_stats *st) {
 	return E_HUFFMAN;
     }
 
-    if (fd->verbose > 1)
-	fprintf(stderr, "Range = %d..%d, nvals=%d, ntot=%d\n",
-		min_val, max_val, nvals, ntot);
-
     // Massive cull of old entropy analysis.
     // If you're interested in code to compare huffman vs beta
     // bit streams, see svn revision r4051
