@@ -84,35 +84,46 @@ typedef struct {
     HashTable *h;
     int nsamp; // total number of values added
     int nvals; // total number of unique values added
+    int64_t min_val, max_val;
 } cram_stats;
 
 /* NB: matches java impl, not the spec */
 enum cram_encoding {
     E_NULL               = 0,
-    E_EXTERNAL           = 1,
-    E_GOLOMB             = 2,
-    E_HUFFMAN            = 3,
+    E_EXTERNAL           = 1, // Only for BYTE type in CRAM 4
+    E_GOLOMB             = 2, // Not in CRAM 4
+    E_HUFFMAN            = 3, // Not in CRAM 4
     E_BYTE_ARRAY_LEN     = 4,
     E_BYTE_ARRAY_STOP    = 5,
-    E_BETA               = 6,
-    E_SUBEXP             = 7,
-    E_GOLOMB_RICE        = 8,
-    E_GAMMA              = 9,
-    E_XHUFFMAN           = 10, // To external block
-    E_XPACK              = 11, // Transform to sub-codec
-    E_XRLE               = 12, // Transform to sub-codec
-    E_XDELTA             = 13, // Transform to sub-codec
-    E_NUM_CODECS, /* Total number of codecs, not a real one. */
+    E_BETA               = 6, // Not in CRAM 4
+    E_SUBEXP             = 7, // Not in CRAM 4
+    E_GOLOMB_RICE        = 8, // Not in CRAM 4
+    E_GAMMA              = 9, // Not in CRAM 4
+
+    // CRAM 4 specific codecs
+    E_VARINT_UNSIGNED    = 41, // Specialisation of EXTERNAL
+    E_VARINT_SIGNED      = 42, // Specialisation of EXTERNAL
+    E_CONST_BYTE         = 43, // Alternative to HUFFMAN with 1 symbol
+    E_CONST_INT          = 44, // Alternative to HUFFMAN with 1 symbol
+
+    // More experimental ideas, not documented in spec yet
+    E_XHUFFMAN           = 50, // To external block
+    E_XPACK              = 51, // Transform to sub-codec
+    E_XRLE               = 52, // Transform to sub-codec
+    E_XDELTA             = 53, // Transform to sub-codec
+
+    // Total number of codecs, not a real one.
+    E_NUM_CODECS,
 };
 
 enum cram_external_type {
     E_INT                = 1,
-    E_LONG               = 2,
+    E_LONG               = 2, // Internal only for CRAM 4; decays to E_INT
     E_BYTE               = 3,
     E_BYTE_ARRAY         = 4,
-    E_BYTE_ARRAY_BLOCK   = 5,
-    E_SINT               = 6, // signed INT
-    E_SLONG              = 7, // signed LONG
+
+    // Internal only.
+    E_BYTE_ARRAY_BLOCK   = 5, // externally this decays to E_BYTE_ARRAY
 };
 
 /* External IDs used by this implementation (only assumed during writing) */
@@ -798,6 +809,7 @@ struct cram_fd;
 typedef struct varint_vec {
     // Returns number of bytes decoded from fd, 0 on error
     int (*varint_decode32_crc)(struct cram_fd *fd, int32_t *val_p, uint32_t *crc);
+    int (*varint_decode32s_crc)(struct cram_fd *fd, int32_t *val_p, uint32_t *crc);
     int (*varint_decode64_crc)(struct cram_fd *fd, int64_t *val_p, uint32_t *crc);
 
     //int (*varint_get32)(const char *cp, const char *endp, int32_t *val_p);
@@ -936,6 +948,8 @@ typedef struct cram_fd {
     // variable integer decoding callbacks.
     // This changed in CRAM4.0 to a data-size agnostic encoding.
     varint_vec vv;
+
+    int level_fixed; // boolean flag to indicate if level is explicit.
 } cram_fd;
 
 #if defined(CRAM_IO_CUSTOM_BUFFERING)
@@ -1067,6 +1081,7 @@ enum cram_option {
     CRAM_OPT_USE_FQZ,
     CRAM_OPT_EMBED_CONS,
     CRAM_OPT_USE_TOK,
+    CRAM_OPT_PROFILE
 };
 
 /* BF bitfields */
